--[[ shepard.lua for OpenTX Version 2.1.9
	by "Miami" Mike Naylor, April 15, 2018
]]
local TimeOut, Events, LastFM = 0, 0, 0
local Wav, SoundSwitch = {}, {}

--[[Add an "Altx" altitude sensor to your Telemetry
page with the same settings as your "Alt" sensor except
set units to "m" or "Raw (-)" and Precision to "2".]]
local Alt = getFieldInfo("Altx").id

local Scale = 10 --Scale sets meters per octave.
--[[To adjust Scale with a global variable, set ScaleGV
to the number of the global variable minus 1. Examples:
local ScaleGV = 0 -- Use GVAR 1
...
local ScaleGV = 8 -- Use GVAR 9
]]
-- local ScaleGV = 0 -- Use GVAR 1 to set meters per octave

local Period = 16 -- period between beeps
local Delay = 1.5 -- seconds to pause for sound event

local function init_func()
	for i = 0, 99 do
		Wav[i] = "/SHEPARD/".. i .. ".wav"
	end
	local sw, swtable = 0, {}
	for i = 0, 7 do -- switches
		local id = i + 92
		for j = 1, 3 do
			sw = 3 * i + j
			swtable[sw] = {}
			swtable[sw].id = id
			swtable[sw].trueval = j - 2
		end
	end
	for i = 0, 31 do -- logical switches
		sw = 51 + i
		swtable[sw] = {}
		swtable[sw].id = i + 100
		swtable[sw].trueval = 1
	end
	local playtrack, playvalue = 11, 12
	for i = 0, 63 do
		local positive = true
		local func = model.getCustomFunction(i).func
		if func == playtrack or func == playvalue then
			sw = model.getCustomFunction(i).switch
			if sw < 0 then
				positive = false
				sw = -sw
			end	
			if swtable[sw] then
				Events = Events + 1
				SoundSwitch[Events] = swtable[sw]
				local id = swtable[sw].id
				SoundSwitch[Events].lastval = getValue(id)/1024
				SoundSwitch[Events].positive = positive
			end
		end
	end
end	

local function run_func()
	local fm = getFlightMode()
	local sc = Scale
	if ScaleGV then
		local gv = model.getGlobalVariable(ScaleGV, fm)
		sc = (gv >= 5 and gv <= 100) and gv or Scale
	end
	local addtime = false
	if fm ~= LastFM then
		addtime = true
		LastFM = fm
	end
	if Events > 0 then
		for i = 1, Events do
			local sf = SoundSwitch[i]
			local currentval = getValue(sf.id) / 1024
			if currentval ~= sf.lastval then
				if currentval == sf.trueval and sf.positive then
					addtime = true
				elseif currentval ~= sf.trueval and not sf.positive then
					addtime = true
				end	
			end	
			SoundSwitch[i].lastval = currentval
		end
	end
	local time = getTime()
	if addtime then
		TimeOut = time + 100 * Delay
	end
	if time > TimeOut then
		local alt = math.floor(.5 + 100 * getValue(Alt) / sc) % 100
		playFile(Wav[alt])
		TimeOut = time + Period	
	end	
end
return {run = run_func, init = init_func}